import logo from './logo.svg';
import './App.css';
import { useEffect, useState } from 'react';
import { render } from '@testing-library/react';

const Spoiler = ({header="+", open=true, children}) => {
  let [isOpen=open, setOpen] = useState()
  return (
    <>
      <header onClick={() => setOpen(!isOpen)}>{header}</header>
      <div style={{visibility: isOpen? 'visible' : 'hidden'}}>
        {children}
      </div>
    </>
  )
}

const RangeInput = ({min=1, max=10, type='text'}) => {
  let [valueLen, setValue] = useState()
  return (
    <div>
      <input 
        type={type} 
        style={{outlineColor: valueLen < min || valueLen > max ? 'red' : 'black'}} 
        onChange={(e) => setValue(e.target.value.length)}
      />
    </div>
  )
}

const PasswordConfirm = ({min=8}) => {
  let [pass='', setPass] = useState()
  let [passRepeat, setPassReapeat] = useState()

  return (
    <div style={{border: '1px solid black', backgroundColor: 'whitesmoke', width: 'fit-content'}}>
      <input 
        type='password'
        style={{
          display: 'block',
          borderColor: pass === passRepeat && 
          pass.length >= min && /[0-9]/.test(pass) && !/[\s]/.test(pass)? 'mediumseagreen': 'red'
        }}
        placeholder='password' 
        onChange={(e) => setPass(e.target.value)}
      />
      <br/>
      <input 
        type='password'
        style={{
          display: 'block',
          borderColor: pass === passRepeat? 'mediumseagreen' : 'red',
          visibility: pass.length >= min && 
          /[0-9]/.test(pass) && !/[\s]/.test(pass)? 'visible' : 'hidden' 
        }}
        placeholder='repeat password'
        onChange={(e) => setPassReapeat(e.target.value)}
      />
    </div>
  )
}

const Timer = ({seconds=10}) => {
  let [time=seconds,  setTime] = useState();
    useEffect(() => {
      const interval = setInterval(()=> time > 0? setTime(time - 1) : clearInterval(interval), 1000)
      return() => {clearInterval(interval)}
    },[time])
  return (
    <strong>
      {Math.floor(((time*1000) / (1000 * 60 * 60)) % 24)} :
      {Math.floor(((time*1000) / (1000 * 60)) % 60)} :
      {Math.floor(((time*1000) / 1000) % 60)}
    </strong>
  )
}

const LCD = ({seconds=10}) =>
  <strong>
    {Math.floor(((seconds*1000) / (1000 * 60 * 60)) % 24)} :
    {Math.floor(((seconds*1000) / (1000 * 60)) % 60)} :
    {Math.floor(((seconds*1000) / 1000) % 60)}
  </strong>

const SecondsTimer = ({seconds}) => <h2>{seconds}</h2>

const TimerControl = ({hour=0, minute=0, second=0}) => {
  let [hr=hour, setHr] = useState()
  let [min=minute, setMin] = useState()
  let [sec=second, setSec] = useState()
  let [active=false, setActive] = useState()

  return (
    <span>
      <input
        type="number"
        min="0" 
        max="30"
        placeholder='hr' 
        value={hr} 
        onChange={(e)=> setHr(+e.target.value < 0 || +e.target.value > +e.target.max ? 0 : +e.target.value)}
      />:
      <input
        type="number"
        min="0" 
        max="59"  
        placeholder='min' 
        value={min}
        onChange={(e)=> setMin(+e.target.value > 59 || e.target.value < 0? 0 : +e.target.value)}
      />:
      <input 
        type="number"
        min="0" 
        max="59"
        placeholder='sec' 
        value={sec} 
        onChange={(e)=> setSec(+e.target.value > 59 || e.target.value < 0? 0 : +e.target.value)}
      />

      { active ? <button onClick={() => setActive(false)}>Stop</button> :
                 <button onClick={() => setActive(true)}>Start</button>}
      <br />
      { active ? <Timer seconds={hr*60*60 + min*60 + sec}/> : ''}
    </span>
  )
}

const TimerContainer = ({seconds=1800, refresh=100, render:Component}) => {
  let [timePassed=refresh, setTimePassed] = useState()
  let [time=seconds*1000, setTime] = useState()
  useEffect(() => {
    const interval = setInterval(()=> {
      if(time > -0) {
        setTime(seconds*1000 - timePassed)
        setTimePassed(timePassed + refresh)
      } else {
        return clearInterval(interval);
      }
    },refresh)
    return() => clearInterval(interval)
  }, [refresh, time, timePassed, setTimePassed])
  
  return(
    <Component seconds={time/1000}/>
  )
}


function App() {
  return (
    <div className="App">
      <Spoiler header={<h1>Заголовок(клацать сюда)</h1>} open>
        Контент 1
        <p>
            лорем ипсум траливали и тп.
        </p>
        <img alt="img" src={logo} width="200" heigt="200"/>
        <h3>Range Input</h3>
        <RangeInput min={2}/>
      </Spoiler>
      <br />

      <h3>Password Confirm</h3>
      <PasswordConfirm />

      <h3>Timer Control</h3>
      <TimerControl second={10}/>

      <h3>Timer</h3>
      <Timer seconds={7200}/>
      
      <h3>Timer Container</h3>
      <TimerContainer render={SecondsTimer}/>
      <TimerContainer render={SecondsTimer} seconds={60} refresh={10}/>
      <TimerContainer render={LCD}/>

    </div>
  );
}

export default App;
